@@ -1,4 +1,4 @@
-/*	$OpenBSD: locking.c,v 1.10 2011/01/10 21:00:50 millert Exp $	*/
+/*	$OpenBSD: locking.c,v 1.14 2020/02/09 14:59:20 millert Exp $	*/
 
 /*
  * Copyright (c) 1996-1998 Theo de Raadt <deraadt@theos.com>
@@ -30,13 +30,15 @@
 
 #include "includes.h"
 
-#include <sys/param.h>
+#include <sys/types.h>
+
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <pwd.h>
 #include <syslog.h>
 #include <time.h>
 #include <unistd.h>
+#include <limits.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -45,7 +47,7 @@
 #include "pathnames.h"
 #include "mail.local.h"
 
-static char lpath[MAXPATHLEN];
+static char lpath[PATH_MAX];
 
 void
 rellock(void)
@@ -56,7 +58,7 @@ rellock(void)
 }
 
 int
-getlock(char *name, struct passwd *pw)
+getlock(const char *name, struct passwd *pw)
 {
 	struct stat sb, fsb;
 	int lfd=-1;
@@ -66,9 +68,8 @@ getlock(char *name, struct passwd *pw)
 	(void)snprintf(lpath, sizeof lpath, "%s/%s.lock",
 	    _PATH_MAILDIR, name);
 
-	if (stat(_PATH_MAILDIR, &sb) == -1)
-		merr(FATAL, "%s: %s", _PATH_MAILDIR, strerror(errno));
-	if ((sb.st_mode & S_IWOTH) == S_IWOTH) {
+	if (stat(_PATH_MAILDIR, &sb) != -1 &&
+	    (sb.st_mode & S_IWOTH) == S_IWOTH) {
 		/*
 		 * We have a writeable spool, deal with it as
 		 * securely as possible.
@@ -93,19 +94,12 @@ getlock(char *name, struct passwd *pw)
 				}
 				goto again;
 			}
-#ifndef O_EXLOCK
-#define O_EXLOCK 0
-#endif
 			if ((lfd = open(lpath, O_CREAT|O_WRONLY|O_EXCL|O_EXLOCK,
 			    S_IRUSR|S_IWUSR)) != -1)
 				break;
-#ifndef O_EXLOCK
-			/* XXX : do something! */
-#endif
 again:
 			if (tries > 10) {
-				merr(NOTFATAL, "%s: %s", lpath,
-				    strerror(errno));
+				mwarn("%s: %s", lpath, strerror(errno));
 				seteuid(0);
 				return(-1);
 			}
@@ -121,6 +115,7 @@ again:
 						seteuid(pw->pw_uid);
 					}
 				}
+				close(lfd);
 			}
 			sleep(1U << tries);
 			tries++;
@@ -136,7 +131,7 @@ again:
 			    S_IRUSR|S_IWUSR)) != -1)
 				break;
 			if (tries > 9) {
-				merr(NOTFATAL, "%s: %s", lpath, strerror(errno));
+				mwarn("%s: %s", lpath, strerror(errno));
 				return(-1);
 			}
 			sleep(1U << tries);
@@ -149,7 +144,7 @@ again:
 void
 baditem(char *path)
 {
-	char npath[MAXPATHLEN];
+	char npath[PATH_MAX];
 	int fd;
 
 	if (unlink(path) == 0)
@@ -161,19 +156,27 @@ baditem(char *path)
 	if (rename(path, npath) == -1)
 		unlink(npath);
 	else
-		merr(NOTFATAL, "nasty spool item %s renamed to %s",
-		    path, npath);
+		mwarn("nasty spool item %s renamed to %s", path, npath);
 	/* XXX if we fail to rename, another attempt will happen later */
 }
 
 void
-merr(int isfatal, const char *fmt, ...)
+mwarn(const char *fmt, ...)
 {
 	va_list ap;
 
 	va_start(ap, fmt);
 	vsyslog(LOG_ERR, fmt, ap);
 	va_end(ap);
-	if (isfatal)
-		exit(1);
+}
+
+void
+merr(int eval, const char *fmt, ...)
+{
+	va_list ap;
+
+	va_start(ap, fmt);
+	vsyslog(LOG_ERR, fmt, ap);
+	va_end(ap);
+	exit(eval);
 }
