@@ -1,4 +1,4 @@
-/*	$OpenBSD: lka_filter.c,v 1.73 2023/07/07 14:52:00 op Exp $	*/
+/*	$OpenBSD: lka_filter.c,v 1.77 2024/05/14 13:38:54 op Exp $	*/
 
 /*
  * Copyright (c) 2018 Gilles Chehade <gilles@poolp.org>
@@ -56,6 +56,7 @@ static int	filter_builtins_data(struct filter_session *, struct filter *, uint64
 static int	filter_builtins_commit(struct filter_session *, struct filter *, uint64_t, const char *);
 
 static void	filter_result_proceed(uint64_t);
+static void	filter_result_report(uint64_t, const char *);
 static void	filter_result_junk(uint64_t);
 static void	filter_result_rewrite(uint64_t, const char *);
 static void	filter_result_reject(uint64_t, const char *);
@@ -201,6 +202,7 @@ static void
 lka_proc_config(struct processor_instance *pi)
 {
 	io_printf(pi->io, "config|smtpd-version|%s\n", SMTPD_VERSION);
+	io_printf(pi->io, "config|protocol|%s\n", PROTOCOL_VERSION);
 	io_printf(pi->io, "config|smtp-session-timeout|%d\n", SMTPD_SESSION_TIMEOUT);
 	if (pi->subsystems & FILTER_SUBSYSTEM_SMTP_IN)
 		io_printf(pi->io, "config|subsystem|smtp-in\n");
@@ -665,6 +667,8 @@ lka_filter_process_response(const char *name, const char *line)
 			filter_result_reject(reqid, parameter);
 		else if (strncmp(response, "disconnect|", 11) == 0)
 			filter_result_disconnect(reqid, parameter);
+		else if (strncmp(response, "report|", 7) == 0)
+			filter_result_report(reqid, parameter);
 		else
 			fatalx("Invalid directive: %s", line);
 	}
@@ -738,7 +742,7 @@ filter_protocol_internal(struct filter_session *fs, uint64_t *token, uint64_t re
 			    filter->name,
 			    param,
 			    filter->config->rewrite);
-			    filter_result_rewrite(reqid, filter->config->rewrite);
+			filter_result_rewrite(reqid, filter->config->rewrite);
 			return;
 		}
 		else if (filter->config->disconnect) {
@@ -964,6 +968,16 @@ filter_result_proceed(uint64_t reqid)
 	m_close(p_dispatcher);
 }
 
+static void
+filter_result_report(uint64_t reqid, const char *param)
+{
+	m_create(p_dispatcher, IMSG_FILTER_SMTP_PROTOCOL, 0, 0, -1);
+	m_add_id(p_dispatcher, reqid);
+	m_add_int(p_dispatcher, FILTER_REPORT);
+	m_add_string(p_dispatcher, param);
+	m_close(p_dispatcher);
+}
+
 static void
 filter_result_junk(uint64_t reqid)
 {
@@ -1641,6 +1655,9 @@ lka_report_smtp_filter_response(const char *direction, struct timeval *tv, uint6
 	case FILTER_PROCEED:
 		response_name = "proceed";
 		break;
+	case FILTER_REPORT:
+		response_name = "report";
+		break;
 	case FILTER_JUNK:
 		response_name = "junk";
 		break;
