@@ -1,4 +1,4 @@
-/*	$OpenBSD: smtp.h,v 1.4 2021/03/05 12:37:32 eric Exp $	*/
+/*	$OpenBSD: smtp.h,v 1.5 2024/06/02 23:26:39 jsg Exp $	*/
 
 /*
  * Copyright (c) 2018 Eric Faurot <eric@openbsd.org>
@@ -87,7 +87,6 @@ void smtp_quit(struct smtp_client *);
 void smtp_sendmail(struct smtp_client *, struct smtp_mail *);
 
 /* callbacks */
-void smtp_verify_server_cert(void *, struct smtp_client *, void *);
 void smtp_require_tls(void *, struct smtp_client *);
 void smtp_ready(void *, struct smtp_client *);
 void smtp_failed(void *, struct smtp_client *, int, const char *);
